indexing

	description:

		"Test features of Erl-G on class HASH_TABLE [ANY, STRING]"

	library: "Erl-G Library"
	copyright: "Copyright (c) 2005, Ilinca Ciupa and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date$"
	revision: "$Revision$"

deferred class ERL_G_TEST_ALIVE_TYPES

inherit

	TS_TEST_CASE

	ERL_SHARED_UNIVERSE
		export {NONE} all end


feature -- Test reflection on generated system

	test_suppliers_alive is
			-- Test that suppliers of alive types are also alive.
		local
			person_type, integer_type, boolean_type, string_type: ERL_TYPE
		do
				-- If PERSON is alive, then STRING, INTEGER, and BOOLEAN should also be alive.
			assert_not_equal ("PERSON type alive", Void, universe.type_by_name ("PERSON"))
			assert_not_equal ("INTEGER type alive", Void, universe.type_by_name ("INTEGER"))
			assert_not_equal ("STRING type alive", Void, universe.type_by_name ("STRING"))
			assert_not_equal ("BOOLEAN type alive", Void, universe.type_by_name ("BOOLEAN"))

				-- If HASH_TABLE [ANY, STRING] is alive, then ARRAY [ANY], ARRAYED_LIST [ANY],
				-- BOOLEAN, HASH_TABLE_CURSOR, INTEGER, MISMATCH_CORRECTOR, MISMATCH_INFORMATION,
				-- PRIMES, REAL, and SPECIAL [ANY] should also be alive.
			assert_not_equal ("HASH_TABLE [ANY, STRING] type alive", Void, universe.type_by_name ("HASH_TABLE [ANY, STRING]"))
			assert_not_equal ("ARRAY [ANY] type alive", Void, universe.type_by_name ("ARRAY [ANY]"))
			assert_not_equal ("ARRAYED_LIST [ANY] type alive", Void, universe.type_by_name ("ARRAYED_LIST [ANY]"))
			assert_not_equal ("HASH_TABLE_CURSOR type alive", Void, universe.type_by_name ("HASH_TABLE_CURSOR"))
			assert_not_equal ("MISMATCH_CORRECTOR type alive", Void, universe.type_by_name ("MISMATCH_CORRECTOR"))
			assert_not_equal ("MISMATCH_INFORMATION type alive", Void, universe.type_by_name ("MISMATCH_INFORMATION"))
			assert_not_equal ("PRIMES type alive", Void, universe.type_by_name ("PRIMES"))
			assert_not_equal ("REAL type alive", Void, universe.type_by_name ("REAL"))
			assert_not_equal ("SPECIAL [ANY] type alive", Void, universe.type_by_name ("SPECIAL [ANY]"))

				-- If ARRAYED_LIST [INTEGER] is alive, then ANY, ARRAY [ANY], ARRAYED_LIST_CURSOR,
				-- BOOLEAN, INTEGER, and SPECIAL [ANY] should also be alive.
			assert_not_equal ("ARRAYED_LIST [INTEGER] type alive", Void, universe.type_by_name ("ARRAYED_LIST [INTEGER]"))
			assert_not_equal ("ARRAY [ANY] type alive", Void, universe.type_by_name ("ARRAY [ANY]"))
			assert_not_equal ("ARRAYED_LIST_CURSOR type alive", Void, universe.type_by_name ("ARRAYED_LIST_CURSOR"))
			assert_not_equal ("ANY type alive", Void, universe.type_by_name ("ANY"))
			assert_not_equal ("SPECIAL [ANY] type alive", Void, universe.type_by_name ("SPECIAL [ANY]"))

				-- If CLIENT is alive, then its suppliers should be alive too.
			assert_not_equal ("CLIENT type alive", Void, universe.type_by_name ("CLIENT"))
			assert_not_equal ("SUPPLIER_1 type alive", Void, universe.type_by_name ("SUPPLIER_1"))
			assert_not_equal ("SUPPLIER_2 type alive", Void, universe.type_by_name ("SUPPLIER_2"))
			assert_not_equal ("SUPPLIER_3 type alive", Void, universe.type_by_name ("SUPPLIER_3"))
			assert_not_equal ("SUPPLIER_4 type alive", Void, universe.type_by_name ("SUPPLIER_4"))
			assert_not_equal ("SUPPLIER_5 type alive", Void, universe.type_by_name ("SUPPLIER_5"))
			assert_not_equal ("SUPPLIER_6 type alive", Void, universe.type_by_name ("SUPPLIER_6"))
			assert_not_equal ("SUPPLIER_7 type alive", Void, universe.type_by_name ("SUPPLIER_7"))
			assert_not_equal ("SUPPLIER_8 type alive", Void, universe.type_by_name ("SUPPLIER_8"))
		end

	test_generic_types_alive is
			-- Test that liveness works correctly for generic types.
		local
			a_type: ERL_TYPE
		do
				-- If CLIENT_OF_GENERIC_TYPE_1 is alive, then GENERIC_TYPE_1 should be alive.
			assert_not_equal ("CLIENT_OF_GENERIC_TYPE_1 type alive", Void, universe.type_by_name ("CLIENT_OF_GENERIC_TYPE_1"))
			assert_not_equal ("GENERIC_TYPE_1 [PRIMES] type alive", Void, universe.type_by_name ("GENERIC_TYPE_1 [PRIMES]"))

				-- If CLIENT_OF_GENERIC_TYPE_2 is alive, then GENERIC_TYPE_2 [TUPLE [FILE_NAME], UC_CHARACTER] are also alive.
			assert_not_equal ("CLIENT_OF_GENERIC_TYPE_2 type alive", Void, universe.type_by_name ("CLIENT_OF_GENERIC_TYPE_2"))
			assert_not_equal ("GENERIC_TYPE_2 [TUPLE [FILE_NAME], UC_CHARACTER] type alive", Void, universe.type_by_name ("GENERIC_TYPE_2 [TUPLE [FILE_NAME], UC_CHARACTER]"))

				-- If CLIENT_OF_GENERIC_TYPE_3 is alive, then GENERIC_TYPE_3 [WIDE_CHARACTER_REF]
				-- should also be alive, and we shouldn't see any creation procedure in GENERIC_TYPE_3 [WIDE_CHARACTER_REF].
			assert_not_equal ("CLIENT_OF_GENERIC_TYPE_3 type alive", Void, universe.type_by_name ("CLIENT_OF_GENERIC_TYPE_3"))
			a_type := universe.type_by_name ("GENERIC_TYPE_3 [WIDE_CHARACTER_REF]")
			assert_not_equal ("GENERIC_TYPE_3 [WIDE_CHARACTER_REF] type alive", Void, a_type)
			assert_equal ("no creation procedures for GENERIC_TYPE_3 [WIDE_CHARACTER_REF]", 0, a_type.creation_procedure_count)
		end

end
